/*
 * Demonstration program for Nano-X graphics.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#define MWINCLUDECOLORS
#include "nano-X.h"

static	GR_WINDOW_ID	w1;		/* id for large window */
static	GR_GC_ID	gc1;		/* graphics context for text */

void errorcatcher();			/* routine to handle errors */

int
main(int argc,char **argv)
{
	GR_EVENT	event;		/* current event */
	GR_IMAGE_ID	id = 0;
	GR_SIZE		w = -1;
	GR_SIZE		h = -1;
	GR_IMAGE_INFO	info;

#if !defined(HAVE_JPEG_SUPPORT) && !defined(HAVE_BMP_SUPPORT) && !defined(HAVE_GIF_SUPPORT)
	printf("Sorry, no image support compiled in\n");
	exit(1);	
#endif

	if(argc < 2) {
		printf("Usage: demo3 <image file> [stretch]\n");
		exit(1);
	}

	if (GrOpen() < 0) {
		fprintf(stderr, "cannot open graphics\n");
		exit(1);
	}
	
	GrSetErrorHandler(errorcatcher);

	w1 = GrNewWindow(GR_ROOT_WINDOW_ID, 10, 10, 400, 400, 4, BLACK, WHITE);

	GrSelectEvents(w1, GR_EVENT_MASK_CLOSE_REQ|GR_EVENT_MASK_EXPOSURE);

	GrMapWindow(w1);

	gc1 = GrNewGC();

	GrSetGCForeground(gc1, WHITE);

	if(argc > 2)
		w = 400, h = 400;
	id = GrLoadImageFromFile(argv[1], 0);
	if (id) {
		GrGetImageInfo(id, &info);
		printf("image is %dx%d\n", info.width, info.height);
	}

	while (1) {
		GrGetNextEvent(&event);
		switch(event.type) {
		case GR_EVENT_TYPE_CLOSE_REQ:
			GrFreeImage(id);
			GrClose();
			exit(0);
			/* no return*/
		case GR_EVENT_TYPE_EXPOSURE:
			/*GrDrawImageFromFile(w1, gc1, 0, 0, w, h, argv[1],0);*/
			GrDrawImageToFit(w1, gc1, 0, 0, w, h, id);
			break;
		}
	}

	return 0;
}



/*
 * Here on an unrecoverable error.
 */
void
errorcatcher(code, name, id)
	GR_ERROR	code;		/* error code */
	GR_FUNC_NAME	name;		/* function name which failed */
	GR_ID		id;		/* resource id */
{
	GrClose();
	fprintf(stderr, "DEMO ERROR: code %d, function %s, resource id %d\n",
		code, name, id);
	exit(1);
}
