/*
 * Copyright (c) 2000 Greg Haerr <greg@censoft.com>
 *
 * Nano-X Draw Library
 */
#define MWINCLUDECOLORS
#include <stdio.h>
#include "nano-X.h"
#include "nxdraw.h"

#define XSIZE_CLOSEBOX	9
#define YSIZE_CLOSEBOX	9

static void
InflateRect(GR_RECT *prc, GR_SIZE dx, GR_SIZE dy)
{
	prc->x -= dx;
	prc->y -= dy;
	prc->width += dx * 2;
	prc->height += dy * 2;
}

void
nxPaintNCArea(GR_DRAW_ID id, int w, int h, GR_CHAR *title, GR_BOOL active)
{
	int		x = 0;
	int		y = 0;
	GR_GC_ID	gc = GrNewGC();
	GR_FONT_ID	fontid;
	GR_RECT		r;

	/* draw 2-line 3d border around window*/
	Draw3dOutset(id, x, y, w, h);
	x += 2; y += 2; w -= 4; h -= 4;

	/* draw 1-line inset inside border*/
	GrSetGCForeground(gc, GrGetSysColor(GR_COLOR_APPWINDOW));
	GrRect(id, gc, x, y, w, h);
	x += 1; y += 1; w -= 2; h -= 2;

	/* fill caption*/
	GrSetGCForeground(gc, 
		GrGetSysColor(active? GR_COLOR_ACTIVECAPTION:
			GR_COLOR_INACTIVECAPTION));
	GrFillRect(id, gc, x, y, w, CYCAPTION);

	/* draw caption text*/
	if (title) {
		GrSetGCForeground(gc,
			GrGetSysColor(active? GR_COLOR_ACTIVECAPTIONTEXT:
				GR_COLOR_INACTIVECAPTIONTEXT));
		GrSetGCUseBackground(gc, GR_FALSE);
		fontid = GrCreateFont(GR_FONT_GUI_VAR, 0, NULL);
		GrSetGCFont(gc, fontid);
		GrText(id, gc, x+4, y-1, title, -1, GR_TFASCII|GR_TFTOP);
		GrDestroyFont(fontid);
	}
	y += CYCAPTION;

	/* draw one line under caption*/
	GrSetGCForeground(gc, GrGetSysColor(GR_COLOR_APPWINDOW));
	GrLine(id, gc, x, y, x+w-1, y);

	/* draw close box*/
	r.x = x + w - CXCLOSEBOX - 2;
	r.y = y - CYCAPTION + 2;
	r.width = CXCLOSEBOX;
	r.height = CYCLOSEBOX;

	Draw3dBox(id, r.x, r.y, r.width, r.height,
		GrGetSysColor(GR_COLOR_BTNHIGHLIGHT),
		GrGetSysColor(GR_COLOR_WINDOWFRAME));
	InflateRect(&r, -1, -1);
	GrSetGCForeground(gc, GrGetSysColor(GR_COLOR_APPWINDOW));
	GrFillRect(id, gc, r.x, r.y, r.width, r.height);

	InflateRect(&r, -1, -1);
	GrSetGCForeground(gc, GrGetSysColor(GR_COLOR_BTNTEXT));
	GrLine(id, gc, r.x, r.y, r.x+r.width-1, r.y+r.height-1);
	GrLine(id, gc, r.x, r.y+r.height-1, r.x+r.width-1, r.y);

#if 0
	/* fill in client area*/
	y++;
	h -= CYCAPTION+1;
	GrSetGCForeground(gc, GrGetSysColor(GR_COLOR_APPWINDOW));
	GrFillRect(id, gc, x, y, w, h);
#endif

	GrDestroyGC(gc);
}

/*
 * Draw3dShadow
 * 	NOINDENT_BLACK	T=white, B=black
 * 	NOINDENT_GRAY	T=white, B=dkgray
 * 	INDENT_BLACK	T=black, B=white
 * 	INDENT_GRAY		T=dkgray, B=white
 *
 *	TTTTTTTTTTTTTT
 *	T             B
 *	T             B
 *	 BBBBBBBBBBBBBB
 */
void
Draw3dShadow(GR_DRAW_ID id,int x,int y,int w,int h,GR_COLOR crTop,
	GR_COLOR crBottom)
{
	GR_GC_ID	gc = GrNewGC();

	GrSetGCForeground(gc, crTop);
	//MoveToEx( hDC, x, y+h-2, NULL);
	//LineTo( hDC, x, y);				/* left side*/
	GrLine(id, gc, x, y+h-2, x, y);			/* left*/
	//LineTo( hDC, x+w-1, y);			/* top side*/
	GrLine(id, gc, x, y, x+w-2, y);			/* top*/

	GrSetGCForeground(gc, crBottom);
	//MoveToEx( hDC, x+w-1, y+1, NULL);
	//LineTo( hDC, x+w-1, y+h-1);			/* right side*/
	GrLine(id, gc, x+w-1, y+1, x+w-1, y+h-2);	/* right*/
	//LineTo( hDC, x, y+h-1);			/* bottom side*/
	GrLine(id, gc, x+w-1, y+h-1, x, y+h-1);		/* bottom*/

	GrDestroyGC(gc);
}

/*
 * Draw3dBox
 *
 *	TTTTTTTTTTTTTTB
 *	T             B
 *	T             B
 *	BBBBBBBBBBBBBBB
 */
void
Draw3dBox(GR_WINDOW_ID id,int x,int y,int w,int h,GR_COLOR crTop,
	GR_COLOR crBottom)
{
	GR_GC_ID	gc = GrNewGC();

	GrSetGCForeground(gc, crTop);
	//MoveToEx( hDC, x, y+h-2, NULL);
	//LineTo( hDC, x, y);				/* left side*/
	GrLine(id, gc, x, y+h-2, x, y+1);		/* left*/
	//MoveToEx( hDC, x, y, NULL);
	//LineTo( hDC, x+w-1, y);			/* top side*/
	GrLine(id, gc, x, y, x+w-2, y);			/* top*/

	GrSetGCForeground(gc, crBottom);
	GrLine(id, gc, x+w-1, y, x+w-1, y+h-2);		/* right*/
	//MoveToEx( hDC, x+w-1, y, NULL);
	//LineTo( hDC, x+w-1, y+h-1);			/* right side*/
	GrLine(id, gc, x+w-1, y+h-1, x, y+h-1);		/* bottom*/
	//LineTo( hDC, x-1, y+h-1);			/* bottom side*/

	GrDestroyGC(gc);
}

/*
 * Draw 2 line deep 3d inset
 */
void
Draw3dInset(GR_DRAW_ID id,int x,int y,int w,int h)
{
	Draw3dBox(id, x, y, w, h,
		GrGetSysColor(GR_COLOR_BTNSHADOW),
		GrGetSysColor(GR_COLOR_BTNHIGHLIGHT));
	++x; ++y; w -= 2; h -= 2;
	Draw3dBox(id, x, y, w, h,
		GrGetSysColor(GR_COLOR_WINDOWFRAME),
		GrGetSysColor(GR_COLOR_3DLIGHT));
}

/*
 * Draw 2 line deep 3d outset
 */
void
Draw3dOutset(GR_DRAW_ID id,int x,int y,int w,int h)
{
	Draw3dBox(id, x, y, w, h,
		GrGetSysColor(GR_COLOR_3DLIGHT),
		GrGetSysColor(GR_COLOR_WINDOWFRAME));
	++x; ++y; w -= 2; h -= 2;
	Draw3dBox(id, x, y, w, h,
		GrGetSysColor(GR_COLOR_BTNHIGHLIGHT),
		GrGetSysColor(GR_COLOR_BTNSHADOW));
}

/*
 * Draw 1 line pushed down rectangle
 */
void
Draw3dPushDown(GR_DRAW_ID id, int x, int y, int w, int h)
{
	Draw3dBox(id, x, y, w, h, GrGetSysColor(GR_COLOR_BTNSHADOW),
		GrGetSysColor(GR_COLOR_BTNSHADOW));
}

/*
 * Draw either 3d up or down depending on state
 */
void
Draw3dUpDownState(GR_DRAW_ID id, int x, int y, int w, int h, GR_BOOL fDown)
{
	if (fDown)
		Draw3dPushDown(id, x, y, w, h);
	else Draw3dOutset(id, x, y, w, h);
}

#if 0
void
Draw3dUpFrame(GR_DRAW_ID id, int l, int t, int r, int b)
{
	RECT	rc;
	HBRUSH	hbr;

	SetRect(&rc, l, t, r, b);
	Draw3dBox(hDC, rc.left, rc.top,
		rc.right-rc.left, rc.bottom-rc.top,
		GrGetSysColor(GR_COLOR_3DLIGHT),
		GrGetSysColor(GR_COLOR_WINDOWFRAME));
	InflateRect(&rc, -1, -1);
	Draw3dBox(hDC, rc.left, rc.top,
		rc.right-rc.left, rc.bottom-rc.top,
		GrGetSysColor(GR_COLOR_BTNHIGHLIGHT),
		GrGetSysColor(GR_COLOR_BTNSHADOW));
	InflateRect(&rc, -1, -1);

	hbr = CreateSolidBrush(GrGetSysColor(GR_COLOR_APPWINDOW));
	FillRect(hDC, &rc, hbr);
	DeleteObject(hbr);
}
#endif
